% ************************************************************************
% This script initiates all parameters, look up tables etc needed for
% simulation of a "parallel" hybrid Simulink model.
%
% The script is subject to continuous development and changes and a good
% understaning of the relation the Simulink program is needed for a useful
% outcome.
%
% Mats Alakla, Fot, January 24, 2016.
%
% ************************************************************************

rho_air = 1.2;  % Air density
grav = 9.81;    % Gravitational constant

% *** Select Drive Cycle, Vehicle type, Fuel=Engine and Battery Size
Drive_cycle = 8;
Vehicle = 4;
Fuel = 2;

% *** Load drive cycles **************************************************
% The following "switch" selects a drive cycle where the first 12
% alternatives are according to an "Lund University standard" and cycle
% 13ff are according to the Volvo VSIM standard. IN any case, the data is
% boiled down to a matrix "cycle" with 6 columns: 1) time [s], 2) speed
% reference [m/s], 3) position [m], 4) altitude [m], 5) slope [rad] and 6)
% slide in availability [-]. The "slide in availability" is in some models
% used to indicathe the presence of an Electric Road, but that is not used
% in the Parallel Hybrid model supplied with this script.

clear cycle
switch Drive_cycle
    case 1 % nedc
        load ..\DriveCycles\nedc;
        cycle = nedc;
    case 2 % us06;
        load ..\DriveCycles\us06;
        cycle = us06;
    case 3 % Bus 1
        load ..\DriveCycles\london159;
        cycle = london159;
    case 4 % Bus 2
        load ..\DriveCycles\cbr85;
        cycle = cbr85;
    case 5 % Distribution 1
        load ..\DriveCycles\Distr1;
        cycle = Distr1;
    case 6 % Long Haul 1
        load ..\DriveCycles\AccCycle;
        cycle = AccCycle;
    case 7 % Long Haul 2
        load ..\DriveCycles\LongHaul2;
        cycle = LongHaul2;
    case 8 % Long Haul 3
        load ..\DriveCycles\LongHaul3;
        cycle = LongHaul3;
    case 9 % Long Haul 4
        load ..\DriveCycles\LongHaul4;
        cycle = LongHaul4;
    case 10 % ramp
        load ..\DriveCycles\Ramp;
        cycle = Ramp;
    case 11 % Landskrona Slide IN Bus
        load ..\DriveCycles\LandskronaSI
        cycle = LAcycle;
    case 12 % SwedenRoute
        load ..\DriveCycles\SwedenRoute
        cycle = SwedenRoute;       
end

endtime = max(cycle(:,1));

cycle(1:length(cycle),6)=0; % make sure that no "Slide In" is indicated

% *** Vehicle Choise *****************************************************
% In this section the main vehicle parameters are selected, for 4 different
% types of vehicles.

switch Vehicle
    case 1 % Car
        Mv = 1600; % Vehicle weight
        rw = 0.30;	        % wheel radius (m)
        Cd = 0.26; 	        % air_resistance
        Cr = 0.008;	        % roll resistance
        Av = 2.55;	        % Front area
        vmax=172/3.6;       % 172 km/h maxhastighet
        Pmax=100000;
        Number_of_gears = 6;
        MinShiftTime = 0.5;
        Paux = 1000;         % Without AC
    case 2 % City Bus (Coman Maxirider)
        Wbatt=78*3.6e6;
        Mv = 5990;       % Vehicle curb weight 
        rw = 0.349;   %0.506;	        % wheel radius (m)
        Cd = 0.79; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
        Cr = 0.00576;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
        Av = 2.23*2.7;   % Front area as Volvo FE6x2
        vmax = 90/3.6;       % max maxhastighet
        Pem_max = 120000;
        Number_of_gears = 2;
        MinShiftTime = 2;
        % Paux = 2000;         % Without AC
    case 3 %  City Bus (Volvo 7900 Electric) 
        Wbatt=76*3.6e6;
        Mv = 19000;         % Vehicle curb weight + 250 kg passenger and cargo
        rw = 0.506;	        % wheel radius (m)
        Cd = 0.79; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
        Cr = 0.00576;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
        Av = 2.55*3.28;   % Front area as Volvo FE6x2
        vmax=90/3.6;       % max maxhastighet
        Pem_max = 160000;
        Number_of_gears = 3;
        MinShiftTime = 2;
        % Paux = 4000;      % Without AC
    case 4 % Tesla Semi Truck
        Wbatt = 3*800*3.6e6;
        Mv = 36287;         % Vehicle curb weight 80000 pounds
        rw = 0.506;	        % wheel radius (m)
        Cd = 0.36; 	        % air_resistance. (Sports Car 0.3-0.4, Ecomony Car 0.4-0.5, Pickup Truck 0.5, Tractor-Trailer,with fairings 0.6-0.7, Tractor-Trailer 0.7-0.9)
        Cr = 0.0032;	    % roll resistance (0.006...0.008 for low roll resist tires. 0.01...0.015 ordinary tires)
        Av = 2.550*3.8;     % Front area as Volvo FE6x2
        vmax = 110/3.6;       % max maxhastighet
        Pem_max = 2*192000; % Traction power per axle
        Number_of_gears = 1;
        Paux = 2000;    % Without AC
end

% ******************** Road and Performance *******************************
Slope_vector = [0 2 5 10]; % [%]
Max_curb_hight = 0.065;  %155;  % [m]
Max_start_slope = 0.05;%
hc=Max_curb_hight;  
Maxle = 0.7*Mv;
Tw_curb = rw*Maxle*grav/sin(acos(-(hc-rw)/rw));
Tw_slope = rw*Mv*grav*sin(atan(Max_start_slope/100));
Tw_roll = Mv*grav*Cr*rw*cos(atan(Max_start_slope/100));
Tw_drag = rw*0.5*Av*Cd*rho_air*vmax^2;

v=[0:vmax*3.6]/3.6;  % [m/s]
figure(20), clf

for i=1:length(Slope_vector),
    Tw_slope = rw*Mv*grav*sin(atan(Slope_vector(i)/100));
    Tw_roll = Mv*grav*Cr*rw*cos(atan(Slope_vector(i)/100));
    Tmax(i,1) = Tw_slope; % Needed to start
    for j=2:length(v)
        Tmax(i,j) = Tw_slope + Tw_roll + rw*0.5*Av*Cd*rho_air*v(j)^2; % Needed to drive and accelerate
    end
    plot(v*3.6,Tmax(i,:))
    hold on
    text(112,1.01*Tmax(i,110),[num2str(Slope_vector(i)) ' %'])
end
xlabel('Speed [km/h]')
ylabel('Wheel torque [Nm]')

% *** Electric machine parameters  ***************************************
% The following lines set the EM drive performance
% Assuming gr1=23:1 and gr2=15:1 as gear ratios, the highest ratio should define
% the maxium Traction Machine Speed. Wwheel_max=vmax/rw. wem_max = gr1*Wwheel_max
gr1=23
gr2=15
wem_max = gr1*vmax/rw

fwr = 3.5; % Field weakening ratio, i.e. ratio between max speed and base
%  speed
wem_min = 0;
% wem_max = 10000*pi/30; % EM max speed
% wem_vmax = 10000*pi/30; % EM Speed at max vehicle speed
Tem_max = Pem_max/(wem_max/fwr);  % Peak continuous torque
% The call for "CreateEMmap" is used for the EM in the same way as the call
% for i "CreateICEmap_special" s used regarding the ICE. See related
% comment for the ICE above. 
[PtoTem,EtaEM,Tem,Wem,Tem_lim,EnergyConsEM] = CreateEMmap(Pem_max,wem_max,Tem_max);

% *** Transmission settings **********************************************

Utvx = wem_max/(vmax/rw); % "Gear one".

EtaGEAR = 0.97; % Simplest possible efficiency model of the transmission

figure(20)
plot(Wem/Utvx*rw*3.6,2*Tem_lim*Utvx,'r');
text(80,2*Tem_lim(1)*Utvx,['Gear ratio = ' num2str(floor(10*Utvx)/10)])
grid

axis([0 120 0 1.2*max(2*Tem_lim*Utvx)])

% legend('Drag Torque','Roll Torque','Slope torque (8%)',['Curb Torque ' num2str(Max_curb_hight) ' [m]'],'Accel Torque ("0..50 in 17 sec")','Max Wheel Torque')
% *** Power Electronics efficiency ***************************************
% Since the simulation program is design NOT to express Voltages and
% Currents, only Powers, the simplest form of PEC efficiency mode is used.

EtaPE = 0.97; % PEC efficiency

% *** Battery parameters *************************************************
% The battery model is a constant voltage in series with a resistance. The
% electric variables voltage and current is recalculated into termina power
% and efficiencies. this is done in the call to "CreateBATTmap".

[EtaBATT,Pbatt,Mbatt,Pbatt_max_new,Wbatt_new]=CreateBATTmap(Pem_max,Wbatt);
SOC_batt_ref_value = 20; % target value for SOC control
SOC_batt_start_value = 90; % Start value of SOC
SOC_tract_max = 95; % Max SOC level, no negative EM power allowed above.
SOC_tract_min = 15; % Min SOC level, no positive EM power allowed below.

% *** Controller parameters **********************************************

% ksoc = max(Wbatt)/400/Tau_charge; % Max SOC control gain with respect to
% % symmetric optimum  
% ksoc_vect = ksoc*tan(pi/2.05*[-1:1/15:1])*2; % Gain vector for SOC control



